/*****************************************************************************

    Bible Reader for Palm
    Copyright (C) 2001  Ho Chuen Poon
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
    You may contact Ho Chuen Poon at poetry@hongkong.com

*****************************************************************************/

#include <PalmOS.h>
#include "PalmBible.h"

static UInt32 creator;

static LockRecordPtr recordStack;

static UInt16 totalVersions;
static LocalID versionID[dmxVersionLimit];
static DmOpenRef versionDB[dmxVersionLimit];
static UInt16 versionCard[dmxVersionLimit];

static MemHandle* versionHandle[dmxVersionLimit];
static void** versionPtr[dmxVersionLimit];

static Boolean dmxOpen; 

static void initCreatorID()
{
	UInt16 card;
	LocalID appDB;

	ErrFatalDisplayIf(SysCurAppDatabase(&card, &appDB),
		"Could not get current app database.");
	ErrFatalDisplayIf(
		DmDatabaseInfo(card, appDB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		&creator),
		"Could not get app database info, looking for creator ID");
}

static void initVersionDB()
{
	DmSearchStateType state;
	UInt16 cardNo, totalRecord, i;
	LocalID dbID = 0;
	Err err;

	err = DmGetNextDatabaseByTypeCreator(true, &state, dmxVersionType, creator, false, &cardNo, &dbID);
	for (totalVersions=0; totalVersions<dmxVersionLimit && !err && dbID; totalVersions++){
		versionID[totalVersions] = dbID;
		versionDB[totalVersions] = DmOpenDatabase(0, dbID, dmModeReadOnly);
		versionCard[totalVersions] = cardNo;

		totalRecord = DmNumRecords(versionDB[totalVersions]);
		versionHandle[totalVersions] = MemPtrNew(sizeof(MemHandle) * totalRecord);
		versionPtr[totalVersions] = MemPtrNew(sizeof(void*) * totalRecord);
		for (i=0; i< totalRecord; i++){
			versionHandle[totalVersions][i] = NULL;
		}

		err = DmGetNextDatabaseByTypeCreator(false, &state, dmxVersionType, creator, false, &cardNo, &dbID);
	}
}

static void unlockVersionRecords(UInt16 i)
{
	UInt16 totalRecord, j;

    totalRecord = DmNumRecords(versionDB[i]);
    for (j=0; j< totalRecord; j++){
        if (versionHandle[i][j] != NULL){
            MemHandleUnlock(versionHandle[i][j]);
            DmReleaseRecord(versionDB[i], j, false);
            versionHandle[i][j] = NULL;
        }
    }
}

static void closeVersionDB()
{
	UInt16 i;
	
	for(i=0; i<totalVersions; i++){
		unlockVersionRecords(i);
	
		MemPtrFree(versionHandle[i]);
		MemPtrFree(versionPtr[i]);
		
		DmCloseDatabase(versionDB[i]);
	}
}

void DmxOpenDatabase()
{
	recordStack = NULL;

	initCreatorID();
	initVersionDB();
	
	dmxOpen = true;
}

void DmxCloseDatabase()
{
	if (dmxOpen){
		closeVersionDB();
		dmxOpen = false;
	}
}

DmOpenRef DmxOpenConfig(char* name)
{
	DmOpenRef dbRef;
	LocalID dbID;

	DmCreateDatabase(0, name, creator, dmxConfigType, false);
	dbID = DmFindDatabase(0, name);
	dbRef = DmOpenDatabase(0, dbID, dmModeReadWrite);

	return dbRef;
}

void DmxDeleteVersion(UInt16 version)
{
	if (!dmxOpen){
		DmDeleteDatabase(versionCard[version - 1], versionID[version - 1]);
	}
}

void DmxUncacheVersion(UInt16 version)
{
	// unlockVersionRecords(version - 1);
}

void* DmxLockVersionRecord(UInt16 version, UInt16 record)
{
	if (versionHandle[version - 1][record] == NULL){
		versionHandle[version - 1][record] = DmGetRecord(versionDB[version - 1], record);
		versionPtr[version - 1][record] = MemHandleLock(versionHandle[version - 1][record]);
	}
	
	return versionPtr[version - 1][record];
}

void DmxUnlockVersionRecord(UInt16 version, UInt16 record)
{
}

void DmxAddRecord(DmOpenRef dbRef, UInt16 recIndex, void* rec, UInt16 size)
{
	MemHandle dbHandle;
	UInt16 dbIndex;
	void* dbPtr;
	
	dbHandle = DmNewRecord(dbRef, &recIndex, size);
	dbPtr = MemHandleLock(dbHandle);

	DmWrite(dbPtr, 0, rec, size);

	MemHandleUnlock(dbHandle);
	DmReleaseRecord(dbRef, recIndex, true);
}

void* DmxLockRecord(DmOpenRef dbRef, UInt16 recIndex)
{
	LockRecordPtr lockRecord;

	lockRecord = MemPtrNew(sizeof(LockRecordType));
	lockRecord->dmxLockRef = dbRef;
	lockRecord->dmxLockIndex = recIndex;
	lockRecord->dmxLockHandle = DmGetRecord(dbRef, recIndex);

	lockRecord->prev = recordStack;

	recordStack = lockRecord;
	
	return MemHandleLock(lockRecord->dmxLockHandle);
}

void DmxUnlockRecord()
{
	LockRecordPtr lockRecord;

	MemHandleUnlock(recordStack->dmxLockHandle);
	DmReleaseRecord(recordStack->dmxLockRef, recordStack->dmxLockIndex, false);

	lockRecord = recordStack->prev;
	MemPtrFree(recordStack);
	recordStack = lockRecord;
}

UInt16 DmxGetTotalVersions()
{
	return totalVersions;
}

